﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace EduNetworkBuilder
{
    public partial class ShapeEditor : Form
    {
        NetShape ShapeForEditing = null;

        public ShapeEditor()
        {
            InitializeComponent();
            LocalSetup();
        }

        public ShapeEditor(NetShape WhatToEdit)
        {
            InitializeComponent();
            ShapeForEditing = WhatToEdit;
            LocalSetup();
        }

        void LocalSetup()
        {

            Network myNet = NB.GetNetwork();
            List<String> ColorNames = myNet.ColorNames;
            ColorNames.Sort();

            cbFillColor.Items.Clear();
            cbLineColor.Items.Clear();
            foreach (string one in ColorNames)
            {
                cbFillColor.Items.Add(one);
                cbLineColor.Items.Add(one);
            }

            lbShapeList.Items.Clear();
            foreach(string one in myNet.ShapeDescriptions())
            {                
                lbShapeList.Items.Add(one);
            }

            cbShape.Items.Clear();
            foreach(NetShapeType one in Enum.GetValues(typeof(NetShapeType)))
            {
                if (one != NetShapeType.none)
                    cbShape.Items.Add(one.ToString());
            }

            tbHeight.Leave += DataUpdated;
            tbWidth.Leave += DataUpdated;
            tbX.Leave += DataUpdated;
            tbY.Leave += DataUpdated;
            cbFillColor.Leave += DataUpdated;
            cbLineColor.Leave += DataUpdated;
            cbShape.Leave += DataUpdated;
        }

        void DataUpdated(object sender, EventArgs e)
        {
            ValuesToShape();
        }

        private void lbShapeList_SelectedIndexChanged(object sender, EventArgs e)
        {
            ChangeSelectionTo(lbShapeList.Text);
        }

        void ChangeSelectionTo(string ToSelect)
        {
            Network myNet = NB.GetNetwork();
            //Fill in the text from the shape

            ShapeForEditing = null;
            foreach (NetShape oneShape in myNet.Shapes)
            {
                if (oneShape.ToString() == ToSelect)
                    ShapeForEditing = oneShape;
            }
            FieldsFromShape(ShapeForEditing);
        }

        private void FieldsFromShape()
        {
            FieldsFromShape(ShapeForEditing);
        }
        private void FieldsFromShape(NetShape theShape)
        {
            ShapeForEditing = theShape;
            if(ShapeForEditing != null)
            {
                tbX.Text = ShapeForEditing.InArea.X.ToString();
                tbY.Text = ShapeForEditing.InArea.Y.ToString();
                tbHeight.Text = ShapeForEditing.InArea.Height.ToString();
                tbWidth.Text = ShapeForEditing.InArea.Width.ToString();
                cbFillColor.Text = ShapeForEditing.FillColor.Name;
                cbLineColor.Text = ShapeForEditing.LineColor.Name;
                cbShape.Text = ShapeForEditing.MyShape.ToString();
            }
            else
            {
                {
                    tbX.Text = "0";
                    tbY.Text = "0";
                    tbHeight.Text = "0";
                    tbWidth.Text = "0";
                    cbFillColor.Text = "Empty";
                    cbLineColor.Text = "Empty";
                    cbShape.Text = "None";
                }
            }
        }

        private void ValuesToShape()
        {
            ValuesToShape(ShapeForEditing);
        }
        private void ValuesToShape(NetShape theShape)
        {
            ShapeForEditing = theShape;
            if (ShapeForEditing != null)
            {
                int X, Y, Width, Height;
                X = ShapeForEditing.InArea.X;
                Y = ShapeForEditing.InArea.Y;
                Width = ShapeForEditing.InArea.Width;
                Height = ShapeForEditing.InArea.Height;
                int.TryParse(tbX.Text, out X);
                int.TryParse(tbY.Text, out Y);
                int.TryParse(tbWidth.Text, out Width);
                int.TryParse(tbHeight.Text, out Height);
                ShapeForEditing.InArea = new Rectangle(X, Y, Width, Height);
                ShapeForEditing.FillColor = Color.FromName(cbFillColor.Text);
                ShapeForEditing.LineColor = Color.FromName(cbLineColor.Text);
                ShapeForEditing.MyShape = NB.TryParseEnum<NetShapeType>(cbShape.Text, NetShapeType.rectangle);
            }

        }

        private void btnDone_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void lbShapeList_MouseUp(object sender, MouseEventArgs e)
        {
            if(lbShapeList.SelectedItem != null)
            {
                //We have an item that we have clicked on
                ChangeSelectionTo(lbShapeList.Text);
            }
            if (e.Button == MouseButtons.Right)
            {
                //We should make a context menu here.
                //  Delete
                //  Move Up
                //  Move Down
                //  Move to top
                //  Move to bottom
                if(ShapeForEditing != null)
                {
                    int index = 0;
                    if (lbShapeList.ContextMenuStrip == null)
                    {
                        lbShapeList.ContextMenuStrip = new ContextMenuStrip();
                    }
                    lbShapeList.ContextMenuStrip.Items.Add(NB.Translate("_Delete"));
                    lbShapeList.ContextMenuStrip.Items[index++].Click += lblShape_Delete;

                }
            }
            //If it is a left mouse button, we simply select the item so we can edit it
        }

        void lblShape_Delete(object sender, EventArgs e)
        {

        }

        private void lbShapeList_MouseDown(object sender, MouseEventArgs e)
        {

        }
    }
}
